
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Utility template functions for arrays.

// Template source file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ArrayUtils.h"
#include "OwnError.h"

template< class A, class T >
T SumArrayElements( A const & a, T const & )
  {
  Integer const nLength = a.length();
  T oSum = 0;
  for( Integer i = 0; i < nLength; i++ )
    {
    oSum += a( i );
    }
  return oSum;
  }

template< class A, class T >
T SumArrayElementsInRange(
  A const & a, T const &,
  Integer iStart,
  Integer iEnd
  )
  {
  Integer const nLength = a.length();
  if( iStart < 0 || iStart >= nLength ||
      iEnd < 0 || iEnd >= nLength )
    {
    OwnError( "SumArrayElementsInRange: index out of range" );
    return T( 0 );
    }
  if( iStart > iEnd )
    {
    // empty range
    return T( 0 );
    }
  T oSum = 0;
  for( Integer i = iStart; i <= iEnd; i++ )
    {
    oSum += a( i );
    }
  return oSum;
  }

template< class T >
Array< T >
GetRangeFromArray(
  Array< T > const & aSrc,
  Integer nStart,
  Integer nEnd
  )
  {
  Integer const nLengthSrc = aSrc.length();
  if( nEnd < nStart )
    {
    // Empty range is represented by setting nEnd smaller than nStart.
    return Array< T >();
    }
  else if( nStart < 0 || nStart >= nLengthSrc ||
           nEnd < 0 || nEnd >= nLengthSrc )
    {
    OwnError( "GetRangeFromArray: index out of range" );
    return Array< T >();
    }
  else if( nStart == 0 && nEnd == nLengthSrc - 1 )
    {
    return aSrc;
    }
  else
    {
    Integer const nLengthNew = nEnd - nStart + 1;
    Array< T > aResult( nLengthNew );
    // Copy elements in range from source array to result array.
    for( Integer i = 0; i < nLengthNew; i++ )
      {
      aResult( i ) = aSrc( nStart + i );
      }
    return aResult;
    }
  }

template< class T >
Array< T >
ConcatenateArrays( Array< T > const & a1, Array< T > const & a2 )
  {
  Integer const nLength1 = a1.length();
  Integer const nLength2 = a2.length();
  Array< T > aResult( nLength1 + nLength2 );
  // Copy elements from source array 1 to the start of result array.
  for( Integer i1 = 0; i1 < nLength1; i1++ )
    {
    aResult( i1 ) = a1( i1 );
    }
  // Copy elements from source array 2 after the elements from 
  // source array 1.
  for( Integer i2 = 0; i2 < nLength2; i2++ )
    {
    aResult( nLength1 + i2 ) = a2( i2 );
    }
  return aResult;
  }

template< class T >
bool
EqualArrays( Array< T > const & a1, Array< T > const & a2 )
  {
  Integer const n1 = a1.length();
  Integer const n2 = a2.length();
  if( n1 != n2 )
    {
    return false;
    }
  else
    {
    for( Integer i = 0; i < n1; i++ )
      {
      if( a1( i ) != a2( i ) )
        {
        return false;
        }
      }
    return true;
    }
  }
