
// Distributed under GNU Lesser General Public License
// See file LICENSE


// HF atomic data for some atoms

// Implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "AtomicData.h"
#include "IntegerArrayUtils.h"
#include "VectorUtils.h"

OCTAVE_EXPORT_FDEF
HFAtomicData GetHeliumHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 1;
  hfatomicdata.nAtomicNumber = 2;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2 );
  hfatomicdata.anPQN = GetIntegerArray( 1 );
  hfatomicdata.anAMQN = GetIntegerArray( 0 );
  hfatomicdata.nSlaterExps = 1;
  hfatomicdata.anJCounts = GetIntegerArray( 1 );
  hfatomicdata.anKCounts = GetIntegerArray( 0 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( 0, 0 ) = GetColumnVector( 1.0 );
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( 0, 0 ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetLithiumHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 2;
  Integer const i1s = 0;
  Integer const i2s = 1;
  hfatomicdata.nAtomicNumber = 3;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 1 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0 );
  hfatomicdata.nSlaterExps = 1;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1 );
  hfatomicdata.anKCounts = GetIntegerArray( 1, 1 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = ColumnVector( ); 
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 0.5 ); 
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( ); 
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetBerylliumHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 2;
  Integer const i1s = 0;
  Integer const i2s = 1;
  hfatomicdata.nAtomicNumber = 4;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 2 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0 );
  hfatomicdata.nSlaterExps = 1;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1 );
  hfatomicdata.anKCounts = GetIntegerArray( 1, 1 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = GetColumnVector( 1.0 ); 
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetNeonHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 3;
  Integer const i1s = 0;
  Integer const i2s = 1;
  Integer const i2p = 2;
  hfatomicdata.nAtomicNumber = 10;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 2, 6 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2, 2 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0, 1 );
  hfatomicdata.nSlaterExps = 3;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1, 3 );
  hfatomicdata.anKCounts = GetIntegerArray( 2, 2, 2 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i1s, i2p ) = GetColumnVector( 6.0 );
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = GetColumnVector( 1.0 ); 
  avCD( i2s, i2p ) = GetColumnVector( 6.0 );
  avCD( i2p, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2p ) = GetColumnVector( 5.0, 0.0, -2.0 / 5.0 );
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i1s, i2p ) = GetColumnVector( 0.0, 1.0 );
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( ); 
  avCE( i2s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i2p, i1s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2p ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetSodiumHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 4;
  Integer const i1s = 0;
  Integer const i2s = 1;
  Integer const i2p = 2;
  Integer const i3s = 3;
  hfatomicdata.nAtomicNumber = 11;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 2, 6, 1 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2, 2, 3 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0, 1, 0 );
  hfatomicdata.nSlaterExps = 3;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1, 3, 1 );
  hfatomicdata.anKCounts = GetIntegerArray( 2, 2, 2, 2 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i1s, i2p ) = GetColumnVector( 6.0 );
  avCD( i1s, i3s ) = GetColumnVector( 1.0 );
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = GetColumnVector( 1.0 ); 
  avCD( i2s, i2p ) = GetColumnVector( 6.0 );
  avCD( i2s, i3s ) = GetColumnVector( 1.0 );
  avCD( i2p, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2p ) = GetColumnVector( 5.0, 0.0, -2.0 / 5.0 );
  avCD( i2p, i3s ) = GetColumnVector( 1.0 ); 
  avCD( i3s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2p ) = GetColumnVector( 6.0 );
  avCD( i3s, i3s ) = ColumnVector( );
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i1s, i2p ) = GetColumnVector( 0.0, 1.0 );
  avCE( i1s, i3s ) = GetColumnVector( 0.5 ); 
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( ); 
  avCE( i2s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i2s, i3s ) = GetColumnVector( 0.5 ); 
  avCE( i2p, i1s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2p ) = ColumnVector( );
  avCE( i2p, i3s ) = GetColumnVector( 0.0, 1.0 / 6.0 ); 
  avCE( i3s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i3s, i3s ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetMagnesiumHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 4;
  Integer const i1s = 0;
  Integer const i2s = 1;
  Integer const i2p = 2;
  Integer const i3s = 3;
  hfatomicdata.nAtomicNumber = 12;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 2, 6, 2 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2, 2, 3 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0, 1, 0 );
  hfatomicdata.nSlaterExps = 3;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1, 3, 1 );
  hfatomicdata.anKCounts = GetIntegerArray( 2, 2, 2, 2 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i1s, i2p ) = GetColumnVector( 6.0 );
  avCD( i1s, i3s ) = GetColumnVector( 2.0 );
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = GetColumnVector( 1.0 ); 
  avCD( i2s, i2p ) = GetColumnVector( 6.0 );
  avCD( i2s, i3s ) = GetColumnVector( 2.0 );
  avCD( i2p, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2p ) = GetColumnVector( 5.0, 0.0, -2.0 / 5.0 );
  avCD( i2p, i3s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2p ) = GetColumnVector( 6.0 );
  avCD( i3s, i3s ) = GetColumnVector( 1.0 );
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i1s, i2p ) = GetColumnVector( 0.0, 1.0 );
  avCE( i1s, i3s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( ); 
  avCE( i2s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i2s, i3s ) = GetColumnVector( 1.0 ); 
  avCE( i2p, i1s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2p ) = ColumnVector( );
  avCE( i2p, i3s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i3s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i3s, i3s ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

OCTAVE_EXPORT_FDEF
HFAtomicData GetArgonHFAtomicData( )
  {
  HFAtomicData hfatomicdata;
  Integer const nOrbitals = 5;
  Integer const i1s = 0;
  Integer const i2s = 1;
  Integer const i2p = 2;
  Integer const i3s = 3;
  Integer const i3p = 4;
  hfatomicdata.nAtomicNumber = 18;
  hfatomicdata.nOrbitals = nOrbitals;
  hfatomicdata.anOccupations = GetIntegerArray( 2, 2, 6, 2, 6 );
  hfatomicdata.anPQN = GetIntegerArray( 1, 2, 2, 3, 3 );
  hfatomicdata.anAMQN = GetIntegerArray( 0, 0, 1, 0, 1 );
  hfatomicdata.nSlaterExps = 3;
  hfatomicdata.anJCounts = GetIntegerArray( 1, 1, 3, 1, 3 );
  hfatomicdata.anKCounts = GetIntegerArray( 2, 2, 3, 2, 3 );
  Array< ColumnVector > avCD( dim_vector( nOrbitals, nOrbitals ) );
  avCD( i1s, i1s ) = GetColumnVector( 1.0 ); 
  avCD( i1s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i1s, i2p ) = GetColumnVector( 6.0 );
  avCD( i1s, i3s ) = GetColumnVector( 2.0 ); 
  avCD( i1s, i3p ) = GetColumnVector( 6.0 );
  avCD( i2s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i2s ) = GetColumnVector( 1.0 ); 
  avCD( i2s, i2p ) = GetColumnVector( 6.0 );
  avCD( i2s, i3s ) = GetColumnVector( 2.0 ); 
  avCD( i2s, i3p ) = GetColumnVector( 6.0 );
  avCD( i2p, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i2p ) = GetColumnVector( 5.0, 0.0, -2.0 / 5.0 );
  avCD( i2p, i3s ) = GetColumnVector( 2.0 ); 
  avCD( i2p, i3p ) = GetColumnVector( 6.0 );
  avCD( i3s, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i3s, i2p ) = GetColumnVector( 6.0 );
  avCD( i3s, i3s ) = GetColumnVector( 1.0 ); 
  avCD( i3s, i3p ) = GetColumnVector( 6.0 );
  avCD( i3p, i1s ) = GetColumnVector( 2.0 ); 
  avCD( i3p, i2s ) = GetColumnVector( 2.0 ); 
  avCD( i3p, i2p ) = GetColumnVector( 6.0 );
  avCD( i3p, i3s ) = GetColumnVector( 2.0 ); 
  avCD( i3p, i3p ) = GetColumnVector( 5.0, 0.0, -2.0 / 5.0 );
  Array< ColumnVector > avCE( dim_vector( nOrbitals, nOrbitals ) );
  avCE( i1s, i1s ) = ColumnVector( ); 
  avCE( i1s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i1s, i2p ) = GetColumnVector( 0.0, 1.0 );
  avCE( i1s, i3s ) = GetColumnVector( 1.0 ); 
  avCE( i1s, i3p ) = GetColumnVector( 0.0, 1.0 );
  avCE( i2s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i2s ) = ColumnVector( ); 
  avCE( i2s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i2s, i3s ) = GetColumnVector( 1.0 ); 
  avCE( i2s, i3p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i2p, i1s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i2p ) = ColumnVector( );
  avCE( i2p, i3s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i2p, i3p ) = GetColumnVector( 1.0, 0.0, 2.0 / 5.0 );
  avCE( i3s, i1s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2s ) = GetColumnVector( 1.0 ); 
  avCE( i3s, i2p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i3s, i3s ) = ColumnVector( ); 
  avCE( i3s, i3p ) = GetColumnVector( 0.0, 1.0  );
  avCE( i3p, i1s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i3p, i2s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i3p, i2p ) = GetColumnVector( 1.0, 0.0, 2.0 / 5.0 );
  avCE( i3p, i3s ) = GetColumnVector( 0.0, 1.0 / 3.0 ); 
  avCE( i3p, i3p ) = ColumnVector( );
  hfatomicdata.avCD = avCD;
  hfatomicdata.avCE = avCE;
  return hfatomicdata;
  }

