
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Basis class

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef Basis_h
#define Basis_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include "BasisMultiIndex.h"
#include "OwnError.h"

class Basis
  {
  private:

  Array< BasisMultiIndex > aIndices;
  bool fValid;

  public:

  Basis()
    : aIndices(), fValid( false )
      {
      }

  Basis( Integer count, Integer** aiData );

  explicit Basis( Array< BasisMultiIndex > const & ami )
    : aIndices( ami ), fValid( true )
      {
      // basis assumed to be correctly formed
      }

  Basis( Basis const & basis )
    : aIndices( basis.aIndices ),
    fValid( basis.fValid )
    {
    }

  Basis& operator = ( Basis const & basis );

  BasisMultiIndex operator ()( Integer p ) const
    {
    if( p >= 0 && p < aIndices.length() )
      {
      return aIndices( p );
      }
    else
      {
      OwnError( "Basis::operator () const: index out of range" );
      return BasisMultiIndex( 0, 0, 0 );
      }
    }

  BasisMultiIndex& operator ()( Integer p )
    {
    if( p >= 0 && p < aIndices.length() )
      {
      return aIndices( p );
      }
    else
      {
      OwnError( "Basis::operator (): index out of range" );
      return aIndices( 0 );
      }
    }

  Integer length() const
    {
    return aIndices.length();
    }

  bool IsValid() const
      {
      return fValid;
      }

  Array< BasisMultiIndex > const & GetIndices() const
      {
      return aIndices;
      }

  Array< BasisMultiIndex > GetIndices()
      {
      return aIndices;
      }

  ScalingIndex GetMinResolution() const;

  ScalingIndex GetMaxResolution() const;

  Array< Integer > GetSofDecomposition() const;

  Array< Integer > GetNsofDecomposition() const;

  Basis ConstructBasisWithZ( Integer iIndexToZ ) const;

  };

#ifdef OCTAVE_DL_COMPILATION
#include "Basis.cc"
#endif

#endif
