
// Distributed under GNU Lesser General Public License
// See file LICENSE


// class BasisPart

// Header file without implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef BasisPart_h
#define BasisPart_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include "OwnError.h"

class BasisPart
  {
  private:

  WaveletType wt;
  ScalingIndex si;
  Array< TranslationIndex > ati;
  
  public:

  BasisPart( 
    WaveletType wt_,
    ScalingIndex si_,
    Array< TranslationIndex > ati_
    )
    : wt( wt_ ), si( si_ ), ati( ati_ )
      {
      }

  TranslationIndex operator ()( Integer p ) const
    {
    if( p >= 0 && p < ati.length() )
      {
      return ati( p );
      }
    else
      {
      OwnError( "BasisPart::operator () const: index out of range" );
      return 0;
      }
    }

  Integer length() const
    {
    return ati.length();
    }

  WaveletType GetWaveletType() const
      {
      return wt;
      }

  ScalingIndex GetResolution() const
      {
      return si;
      }

  Array< TranslationIndex > GetTranslationIndices() const
      {
      return ati;
      }

  };

#endif
