
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Choosing sign for a wavefunction represented
// in uniform scaling function basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ChooseSignForWavefunction.h"
#include "NTwoToIntegerPower.h"
#include "OwnError.h"
#include <cmath>

Integer
ChooseSignForWavefunction(
  UniformBasis const & usfb,
  ColumnVector vWfScf,
  Real rSignCheckPosition
  )
  {
  Integer const iSign =
    Integer(
      ceil( rSignCheckPosition * NTwoToIntegerPower( usfb.si ) )
      );
  if( iSign < usfb.tiMin || iSign > usfb.tiMax )
    {
    OwnError( "ChooseSignForWavefunction: index out of range" );
    return 1;
    }
  if( usfb.tiMin > usfb.tiMax )
    {
    // sign +1 for empty range
    return 1;
    }
  return ( vWfScf( iSign - usfb.tiMin ) >= 0 ) ? 1 : (-1);
  }
