
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of the expected value of an operator

// Implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ComputeExpectedValue.h"
#include "GetNonnegativePart.h"
#include "ConvertVectorToNearestScfBasis.h"
#include "NTwoToIntegerPower.h"
#include "VectorUtils.h"

OCTAVE_EXPORT_FDEF
Real
ComputeExpectedValue(
  WaveletFilters const & wv,
  Basis const & basisNsof,
  Array< Integer > const & anDecNsof,
  UniformBasis usfbScfBasis, 
  Matrix mxOp, 
  ColumnVector vWf
  )
  {
  ColumnVector const vWfScf =
    ConvertVectorToNearestScfBasis( wv, basisNsof, anDecNsof, usfbScfBasis, vWf );
  pair< UniformBasis, ColumnVector > const usfb_vNonneg =
    GetNonnegativePart(
      pair< UniformBasis, ColumnVector >( usfbScfBasis, vWfScf )
      );
  ColumnVector const vWfNonneg = usfb_vNonneg.second;
  ColumnVector const vWfProd = mxOp * vWf;
  ColumnVector const vWfProdScf =
    ConvertVectorToNearestScfBasis( wv, basisNsof, anDecNsof, usfbScfBasis, vWfProd );
  pair< UniformBasis, ColumnVector > const usfb_vProdNonneg =
    GetNonnegativePart(
      pair< UniformBasis, ColumnVector >( usfbScfBasis, vWfProdScf )
      );
  ColumnVector const vWfProdNonneg = usfb_vProdNonneg.second;
  Real const rDot = Dot( vWfNonneg, vWfProdNonneg );
  return rDot / NTwoToIntegerPower( usfbScfBasis.si );
  }
