
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of HF basis data

// Header file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ComputeHFBasisData.h"
#include "ConstructBasisNsof.h"
#include "ConstructScfBasis.h"

OCTAVE_EXPORT_FDECL
HFBasisData ComputeHFBasisData(
  WaveletFilters const & wv,
  Basis const & basis,
  Integer iIndexToZ
  )
  {
  HFBasisData hfbasisdata;
  hfbasisdata.basisSof = basis;
  hfbasisdata.iIndexToZ = iIndexToZ;
  hfbasisdata.anDecSof = basis.GetSofDecomposition();
  pair< Basis, Array< Integer > > basis_decNsof =
    ConstructBasisNsof( basis, wv.nM );
  hfbasisdata.basisNsof = basis_decNsof.first;
  hfbasisdata.anDecNsof = basis_decNsof.second;
  hfbasisdata.usfbScfBasis =
    ConstructScfBasis( hfbasisdata.basisNsof, hfbasisdata.anDecNsof, wv.nM );
  Basis const basisZSof = basis.ConstructBasisWithZ( iIndexToZ );
  pair< Basis, Array< Integer > > basis_decZNsof =
    ConstructBasisNsof( basisZSof, wv.nM );
  hfbasisdata.basisZNsof = basis_decZNsof.first;
  hfbasisdata.anDecZNsof = basis_decZNsof.second;
  UniformBasis const usfbScfZBasis =
    ConstructScfBasis( hfbasisdata.basisZNsof, hfbasisdata.anDecZNsof, wv.nM );
  if( hfbasisdata.usfbScfBasis != usfbScfZBasis )
    {
    OwnError( "ComputeHFBasisData: internal error" );
    return hfbasisdata;
    }
  return hfbasisdata;
  }
