
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of a norm of real matrix.

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeMatrixNorm.h"
#include "OwnError.h"
// liboctave headers
#include "dbleSVD.h"
// C++ standard headers
#include <cmath>

// Row and column vectors may not be handled correctly.
Real ComputeMatrixNorm( Matrix mx, EnumMatrixNormType nMatrixNormType )
  {
  Integer const nRows = mx.rows();
  Integer const nColumns = mx.columns();
  if( nRows == 0 || nColumns == 0 )
    {
    return 0.0;
    }
  else
    {
    if( nMatrixNormType == mxnormOne )
      {
      return mx.abs().sum().row( 0 ).max();
      }
    else if( nMatrixNormType == mxnormTwo )
      {
      return ( SVD( mx, SVD::sigma_only ).singular_values() )( 0, 0 );
      }
    else if( nMatrixNormType == mxnormInf )
      {
      return mx.transpose().abs().sum().row( 0 ).max();
      }
    else if( nMatrixNormType == mxnormFrob )
      {
      Matrix const mxAbs = mx.abs();
      return
        sqrt( ( product( mxAbs, mxAbs ).sum().sum() )( 0, 0 ) );
      }
    else
      {
      OwnError( "ComputeMatrixNorm: internal error" );
      return -1.0;
      }
    }
  }
