
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of mirror operator
//
//   (Qf)(r) = f(-r)
//

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeMirrorOperator.h"
#include "OwnError.h"
#include <cmath>

static inline
BasisMultiIndex MirrorBasisFunction( BasisMultiIndex beta )
  {
  if( beta.wt == 0 )
    {
    return BasisMultiIndex( beta.wt, beta.si, -beta.ti );
    }
  else if( beta.wt == 1 )
    {
    return BasisMultiIndex( beta.wt, beta.si, -beta.ti - 1 );
    }
  else
    {
    OwnError( "MirrorBasisFunction: invalid basis function type" );
    return BasisMultiIndex( 0, 0, 0 );
    }
  }

static inline
Real ComputeMatrixElement(
  BasisMultiIndex alpha,
  BasisMultiIndex beta
  )
  {
  BasisMultiIndex const beta_mirror =
    MirrorBasisFunction( beta );
  return
    ( ( ( alpha.wt == beta_mirror.wt ) &&
        ( alpha.si == beta_mirror.si ) &&
        ( alpha.ti == beta_mirror.ti ) ) &&
      ( ( alpha.wt != beta.wt ) ||
        ( alpha.si != beta.si ) ||
        ( alpha.ti != beta.ti ) ) )
    ? 1.0
    : 0.0;
  }

Matrix ComputeMirrorOperator(
  Basis const & basis
  )
  {
  if( !basis.IsValid() )
    {
    OwnError( "ComputeMirrorOperator: error in basis" );
    return Matrix( 0, 0 );
    }
  Integer const n = basis.length();
  Matrix op( n, n );
  op.fill( 1E30 );
  for( Integer p = 0; p < n; p++ )
    for( Integer q = 0; q < n; q++ )
      {
      op( p, q ) = ComputeMatrixElement( basis( p ), basis( q ) );
      }
  return op;
  }
