
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of inner product defined as integral
// of the product of functions.

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeNormalInnerProduct.h"
#include "GetNonnegativePart.h"
#include "NTwoToIntegerPower.h"

Real ComputeNormalInnerProduct(
  UniformBasis const & usfb,
  ColumnVector vFn1,
  ColumnVector vFn2
  )
  {
  pair< UniformBasis, ColumnVector > const usfb_vFnNonneg1 =
    GetNonnegativePart(
      pair< UniformBasis, ColumnVector >(
        usfb, vFn1
        )
      );
  pair< UniformBasis, ColumnVector > const usfb_vFnNonneg2 =
    GetNonnegativePart(
      pair< UniformBasis, ColumnVector >(
        usfb, vFn2
        )
      );
  ColumnVector const & vFnNonneg1 = usfb_vFnNonneg1.second;
  ColumnVector const & vFnNonneg2 = usfb_vFnNonneg2.second;
  return
    ( vFnNonneg1.transpose() * vFnNonneg2 ) 
    / NTwoToIntegerPower( usfb.si );
  }
