
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of normalization coefficient for
// a wavefunction in uniform scaling function basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeNormalizationCoefficient.h"
#include "GetNonnegativePart.h"
#include "NTwoToIntegerPower.h"
#include <cmath>

Real
ComputeNormalizationCoefficient(
  UniformBasis const & usfb,
  ColumnVector vWfInUsfb
  )
  {
  pair< UniformBasis, ColumnVector > const usfbNonneg_vNonneg =
    GetNonnegativePart(
      pair< UniformBasis, ColumnVector >(
        usfb,
        vWfInUsfb
        )
      );
  UniformBasis const & usfbNonneg = usfbNonneg_vNonneg.first;
  ColumnVector const & vNonneg = usfbNonneg_vNonneg.second;
  return
    sqrt(
      vNonneg.transpose() * vNonneg /
      NTwoToIntegerPower( usfbNonneg.si )
      );
  }
