
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of a SOF block of an operator
// from a NSOF block.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeOperatorSofPart.h"
#include "BasisPart.h"
#include "GetBasisPartNsof.h"
#include "ComputeTransform.h"

// #define DEBUG_OUTPUT

#ifdef DEBUG_OUTPUT
#include <iostream>
#endif

Matrix ComputeOperatorSofPart(
  WaveletFilters const & wv,
  Matrix mxNsof,
  Basis const & basisnsof,
  Array< Integer > aiDecNsof,
  ScalingIndex siMin,
  WaveletType wtSource,
  ScalingIndex siSource,
  Array< TranslationIndex > atiSource,
  WaveletType wtTarget,
  ScalingIndex siTarget,
  Array< TranslationIndex > atiTarget
  )
  {
#ifdef DEBUG_OUTPUT
  cout << "ComputeOperatorSofPart ENTER" << endl;
#endif
  if( siSource == siTarget )
    {
#ifdef DEBUG_OUTPUT
    cout << "ComputeOperatorSofPart EXIT 1" << endl;
    cout << "  rows = " << mxNsof.rows()
         << ", columns = " << mxNsof.columns()
         << endl;
#endif
    return mxNsof;
    }
  else if( siSource < siTarget )
    {
    ScalingIndex const siNew = siSource + 1;
    BasisPart const amiSNew = GetBasisPartNsof(
      basisnsof, aiDecNsof, siMin, 0, siNew
      );
    Array< TranslationIndex > const atiNew = amiSNew.GetTranslationIndices();
    Matrix const mxFt =
      ( wtSource == 0 )
      ? ComputeTransform( atiSource, atiNew, wv.vHt )
      : ComputeTransform( atiSource, atiNew, wv.vGt );
    Matrix const mxNext =
      ComputeOperatorSofPart(
        wv,
        mxNsof,
        basisnsof,
        aiDecNsof,
        siMin,
        0, siNew, atiNew,
        wtTarget, siTarget, atiTarget
        );
#ifdef DEBUG_OUTPUT
    cout << "ComputeOperatorSofPart EXIT 2" << endl;
    cout << "  rows(Ft) = " << mxFt.rows()
         << ", columns(Ft) = " << mxFt.columns()
         << endl;
    cout << "  rows(Next) = " << mxNext.rows()
         << ", columns(Next) = " << mxNext.columns()
         << endl;
#endif
    return mxFt * mxNext;
    }
  else
    {
    ScalingIndex const siNew = siTarget + 1;
    BasisPart const amiSNew = GetBasisPartNsof(
      basisnsof, aiDecNsof, siMin, 0, siNew
      );
    Array< TranslationIndex > const atiNew = amiSNew.GetTranslationIndices();
    Matrix const mxF =
      ( wtTarget == 0 )
      ? ComputeTransform( atiTarget, atiNew, wv.vH )
      : ComputeTransform( atiTarget, atiNew, wv.vG );
    Matrix const mxNext =
      ComputeOperatorSofPart(
        wv,
        mxNsof,
        basisnsof,
        aiDecNsof,
        siMin,
        wtSource, siSource, atiSource,
        0, siNew, atiNew
        );
#ifdef DEBUG_OUTPUT
    cout << "ComputeOperatorSofPart EXIT 3" << endl;
    cout << "  rows(Next) = " << mxNext.rows()
         << ", columns(Next) = " << mxNext.columns()
         << endl;
    cout << "  rows(F) = " << mxF.rows()
         << ", columns(F) = " << mxF.columns()
         << endl;
#endif
    return mxNext * mxF.transpose();
    }
  }
