
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of wavefunctions in scf basis

// Implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ComputeScfWavefunctions.h"
#include "ConvertVectorToNearestScfBasis.h"
#include "MatrixUtils.h"
#include "OwnError.h"

OCTAVE_EXPORT_FDEF
Matrix
ComputeScfWavefunctions(
  WaveletFilters const & wv,
  HFBasisData const & hfbasisdata,
  Matrix mxWf
  )
  {
  Integer const nOrbitals = mxWf.columns( );
  Integer const nSize =
    hfbasisdata.usfbScfBasis.tiMax - hfbasisdata.usfbScfBasis.tiMin + 1;
  if( nSize < 0 )
    {
    OwnError( "ComputeScfWavefunctions: error in scaling function basis" );
    return Matrix( );
    }
  Matrix mxWfScf(
    nSize,
    nOrbitals,
    0.0
    );
  for( Integer i = 0; i < nOrbitals; i++ )
    {
    ColumnVector const vWfScf =
      ConvertVectorToNearestScfBasis(
        wv, hfbasisdata.basisNsof, hfbasisdata.anDecNsof,
        hfbasisdata.usfbScfBasis,
        mxWf.column( i )
        );
    SetMatrixColumn( mxWfScf, i, vWfScf );
    }
  return mxWfScf;
  }
