
// Distributed under GNU Lesser General Public License
// See file LICENSE

// Computation of a transformation matrix.

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ComputeTransform.h"
#include "OwnError.h"

Matrix ComputeTransform(
  Array< TranslationIndex > aiTarget,
  Array< TranslationIndex > aiSource,
  ColumnVector filter
  )
  {
  if( ( filter.length() <= 0 ) ||
      ( filter.length() % 2 == 0 ) )
    {
    OwnError( "ComputeTransform: error in filter" );
    return Matrix( 0, 0 );
    }
  Integer const m = ( filter.length() - 1 ) / 2;
  Integer const rows = aiTarget.length();
  Integer const columns = aiSource.length();
  Matrix transform( rows, columns );
  transform.fill( 0.0 );
  for( Integer p = 0; p < rows; p++ )
    {
    for( Integer q = 0; q < columns; q++ )
      {
      TranslationIndex i1 = aiTarget( p );
      TranslationIndex i2 = aiSource( q );
      TranslationIndex j = i2 - 2 * i1;
      if( j >= -m && j <= m )
	{
	transform( p, q ) = filter( m + j ); 
	}
      }
    }
  return transform;
  }
