
// Distributed under GNU Lesser General Public License
// See file LICENSE


// A console Helium HF output handler

// Header file without implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ConsoleHeliumHFOutputHandler1.h"
#include "BinaryIO.h"

ConsoleHeliumHFOutputHandler1::ConsoleHeliumHFOutputHandler1( )
  : rEnergy( 0.0 )
  {
  }

void ConsoleHeliumHFOutputHandler1::SetRunName( string sRunName1 )
  {
  sRunName = sRunName1;
  }

void ConsoleHeliumHFOutputHandler1::SetBasisData( HFBasisData const & hfbasisdata1 )
  {
  hfbasisdata = hfbasisdata1;
  }

void ConsoleHeliumHFOutputHandler1::Start( )
  {
  cout.precision( 15 );
  cout.setf( ios::scientific );
  cout << "Computation " << sRunName << " starts." << endl;
  }

void ConsoleHeliumHFOutputHandler1::StartInitialization( )
  {
  }

void ConsoleHeliumHFOutputHandler1::EndInitialization( )
  {
  cout << "Scaling function basis: { "
       << hfbasisdata.usfbScfBasis.wt << ", "
       << hfbasisdata.usfbScfBasis.si << ", "
       << hfbasisdata.usfbScfBasis.tiMin << ", "
       << hfbasisdata.usfbScfBasis.tiMax << " }"
       << endl;
  }

void ConsoleHeliumHFOutputHandler1::StartComputation( )
  {
  }

void ConsoleHeliumHFOutputHandler1::EndComputation( )
  {
  }

void ConsoleHeliumHFOutputHandler1::NotifyResults(
  HeliumHFComputationResults const & oResults1
  )
  {
  oResults = oResults1;
  }

void ConsoleHeliumHFOutputHandler1::NotifyScfWavefunction(
  ColumnVector vWfScf1
  )
  {
  vWfScf = vWfScf1;
  }

void ConsoleHeliumHFOutputHandler1::NotifyScfHydrWavefunction(
  ColumnVector vWfHydrScf1
  )
  {
  vWfHydrScf = vWfHydrScf1;
  }

void ConsoleHeliumHFOutputHandler1::NotifyEnergy(
  Real rEnergy1, ColumnVector vJTerms1, ColumnVector vKTerms1
  )
  {
  rEnergy = rEnergy1;
  vJTerms = vJTerms1;
  vKTerms = vKTerms1;
  }

void ConsoleHeliumHFOutputHandler1::Report( )
  {  
  cout << "Error indicator = " << Integer( oResults.errhf ) << endl;
  if( oResults.errhf == errhfNoError )
    {
    cout << "Convergence obtained in HF iteration." << endl;
    cout << "Energy of the atom = " << rEnergy << endl;
    cout << "epsilon_1s = " << oResults.rEpsilon1s << endl;
    cout << "epsilon_1s_hydr = " << oResults.rEpsilon1sHydr << endl;
    }
  else if( oResults.errhf == errhfWavefunctionNotFound )
    {
    cout << "Wavefunction not found." << endl;
    }
  else if( oResults.errhf == errhfNoConvergence )
    {
    cout << "Convergence not obtained in HF iteration." << endl;
    }
  }

void ConsoleHeliumHFOutputHandler1::Save( )
  {
  if( oResults.errhf == errhfNoError )
    {
    OutputMatrixBin(
      sRunName + "Wf1s.dat", Matrix( oResults.vWf1s )
      );
    OutputMatrixBin(
      sRunName + "HydrWf1s.dat", Matrix( oResults.vWf1sHydr )
      );
    OutputMatrixBin(
      sRunName + "WfScf1s.dat", Matrix( vWfScf )
      );
    OutputMatrixBin(
      sRunName + "HydrWfScf1s.dat", Matrix( vWfHydrScf )
      );
    }
  OutputMatrixBin(
    sRunName + "OpLaplacian.dat", oResults.mxOpLaplacian 
    );
  OutputMatrixBin(
    sRunName + "OpR.dat", oResults.mxOpR
    );
  OutputMatrixBin(
    sRunName + "OpInvR.dat", oResults.mxOpInvR
    );
  OutputMatrixBin(
    sRunName + "OpZR.dat", oResults.mxOpZR
    );
  OutputMatrixBin(
    sRunName + "OpZIntZero.dat", oResults.mxOpZIntZero
    );
  OutputMatrixBin(
    sRunName + "OpZIntInf.dat", oResults.mxOpZIntInf
    );
  OutputMatrixBin(
    sRunName + "OpZSlater0.dat", oResults.mxOpZSlater0
    );
  OutputMatrixBin(
    sRunName + "OpJ1s.dat", oResults.mxOpJ1s
    );
  }

void ConsoleHeliumHFOutputHandler1::Done( )
  {
  cout << "Computation " << sRunName << " ends." << endl;
  }

