
// Distributed under GNU Lesser General Public License
// See file LICENSE


// A console Helium HF output handler

// Header file without implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ConsoleNNHFOutputHandler1.h"
#include "BinaryIO.h"

ConsoleNNHFOutputHandler1::ConsoleNNHFOutputHandler1( )
  : rEnergy( 0.0 )
  {
  }

void ConsoleNNHFOutputHandler1::SetRunName( string sRunName1 )
  {
  sRunName = sRunName1;
  }

void ConsoleNNHFOutputHandler1::SetAtomicData( HFAtomicData const & hfatomicdata1 )
  {
  hfatomicdata = hfatomicdata1;
  }

void ConsoleNNHFOutputHandler1::SetBasisData( HFBasisData const & hfbasisdata1 )
  {
  hfbasisdata = hfbasisdata1;
  }

void ConsoleNNHFOutputHandler1::Start( )
  {
  cout.precision( 15 );
  cout.setf( ios::scientific );
  cout << "Computation " << sRunName << " starts." << endl;
  }

void ConsoleNNHFOutputHandler1::StartInitialization( )
  {
  }

void ConsoleNNHFOutputHandler1::EndInitialization( )
  {
  cout << "Scaling function basis: { "
       << hfbasisdata.usfbScfBasis.wt << ", "
       << hfbasisdata.usfbScfBasis.si << ", "
       << hfbasisdata.usfbScfBasis.tiMin << ", "
       << hfbasisdata.usfbScfBasis.tiMax << " }"
       << endl;
  }

void ConsoleNNHFOutputHandler1::StartComputation( )
  {
  }

void ConsoleNNHFOutputHandler1::EndComputation( )
  {
  }

void ConsoleNNHFOutputHandler1::NotifyResults(
  NondiagNeglHFComputationResults const & oResults1
  )
  {
  oResults = oResults1;
  }

void ConsoleNNHFOutputHandler1::NotifyEnergy(
  Real rEnergy1, ColumnVector vJTerms1, ColumnVector vKTerms1
  )
  {
  rEnergy = rEnergy1;
  vJTerms = vJTerms1;
  vKTerms = vKTerms1;
  }

void ConsoleNNHFOutputHandler1::NotifyScfWavefunctions(
  Matrix mxWfScf1
  )
  {
  mxWfScf = mxWfScf1;
  }

void ConsoleNNHFOutputHandler1::NotifyScfHydrWavefunctions(
  Matrix mxHydrWfScf1
  )
  {
  mxHydrWfScf = mxHydrWfScf1;
  }

void ConsoleNNHFOutputHandler1::NotifyInnerProducts(
  Matrix mxInnerProducts1
  )
  {
  mxInnerProducts = mxInnerProducts1;
  }

void ConsoleNNHFOutputHandler1::Report( )
  {
  Integer const nOrbitals = hfatomicdata.anOccupations.length( );
  cout << "Error indicator = " << Integer( oResults.errhf ) << endl;
  if( oResults.errhf == errhfNoError )
    {
    cout << "Convergence obtained in HF iteration." << endl;
    cout << "Energy of the atom = " << rEnergy << endl;
    cout << "Diagonal Lagrange multipliers:" << endl;
    for( Integer i1 = 0;
         i1 < oResults.vDiagLagrangeMultipliers.length();
         i1++ )
      {
      cout << " " << oResults.vDiagLagrangeMultipliers( i1 );
      }
    cout << endl;
    cout << "Hydrogenic diagonal Lagrange multipliers:" << endl;
    for( Integer i1 = 0;
         i1 < oResults.vHydrDiagLagrangeMultipliers.length();
         i1++ )
      {
      cout << " " << oResults.vHydrDiagLagrangeMultipliers( i1 );
      }
    cout << endl;
    cout << "Integrals of products of radial wavefunctions:" << endl;
    for( Integer i1 = 0; i1 < nOrbitals; i1++ )
      {
      for( Integer i2 = i1; i2 < nOrbitals; i2++ )
        {
	cout << " " 
             << hfatomicdata.anPQN( i1 ) << ", " 
             << hfatomicdata.anAMQN( i1 ) << " (" << i1 << ") ; "
	     << hfatomicdata.anPQN( i2 ) << ", " 
             << hfatomicdata.anAMQN( i2 ) << " (" << i2 << ") = ";
	cout << mxInnerProducts( i1, i2 ) << endl;
        }
      }
    }
  else if( oResults.errhf == errhfWavefunctionNotFound )
    {
    cout << "Error in orbital "
         << oResults.nErrorInOrbital
         << endl;
    }
  else if( oResults.errhf == errhfNoConvergence )
    {
    cout << "Convergence not obtained in HF iteration." << endl;
    }
  }

void ConsoleNNHFOutputHandler1::Save( )
  {
  if( oResults.errhf == errhfNoError )
    {
    OutputMatrixBin(
      sRunName + "DLM.dat", Matrix( oResults.vDiagLagrangeMultipliers )
      );
    OutputMatrixBin(
      sRunName + "HydrDLM.dat",
      Matrix( oResults.vHydrDiagLagrangeMultipliers )
      );
    OutputMatrixBin(
      sRunName + "Wavefunctions.dat", oResults.mxWavefunctions
      );
    OutputMatrixBin(
      sRunName + "HydrWavefunctions.dat", oResults.mxHydrWavefunctions
      );
    OutputMatrixBin(
      sRunName + "WfScf.dat", mxWfScf
      );
    OutputMatrixBin(
      sRunName + "HydrWfScf.dat", mxHydrWfScf
      );
    OutputMatrixBin(
      sRunName + "IP.dat", mxInnerProducts
      );
    }
  OutputMatrixBin(
    sRunName + "OpLaplacian.dat", oResults.mxOpLaplacian 
    );
  OutputMatrixBin(
    sRunName + "OpR.dat", oResults.mxOpR
    );
  OutputMatrixBin(
    sRunName + "OpInvR.dat", oResults.mxOpInvR
    );
  OutputMatrixBin(
    sRunName + "OpZR.dat", oResults.mxOpZR
    );
  OutputMatrixBin(
    sRunName + "OpZIntZero.dat", oResults.mxOpZIntZero
    );
  OutputMatrixBin(
    sRunName + "OpZIntInf.dat", oResults.mxOpZIntInf
    );
  if( oResults.amxOpsZSlater.length( ) > 0 )
    {
    Integer const n = oResults.amxOpsZSlater( 0 ).columns( );
    Matrix mxOpsZSlater( 0, n );
    for( Integer i1 = 0; i1 < oResults.amxOpsZSlater.length(); i1++ )
      {
      mxOpsZSlater = mxOpsZSlater.stack( oResults.amxOpsZSlater( i1 ) );
      }
    OutputMatrixBin(
      sRunName + "OpsZSlater.dat", mxOpsZSlater
      );
    }
  if( oResults.amxOpsDirectInt.length( ) > 0 )
    {
    Integer const n = oResults.amxOpsDirectInt( 0 ).columns( );
    Matrix mxOpsDirectInt( 0, n );
    for( Integer i2 = 0; i2 < oResults.amxOpsDirectInt.length(); i2++ )
      {
      mxOpsDirectInt =
        mxOpsDirectInt.stack( oResults.amxOpsDirectInt( i2 ) );
      }
    OutputMatrixBin(
      sRunName + "OpsDirectInt.dat", mxOpsDirectInt
      );
    }
  if( oResults.amxOpsExchangeInt.length( ) > 0 )
    {
    Integer const n = oResults.amxOpsExchangeInt( 0 ).columns( );
    Matrix mxOpsExchangeInt( 0, n );
    for( Integer i3 = 0; i3 < oResults.amxOpsExchangeInt.length(); i3++ )
      {
      mxOpsExchangeInt =
        mxOpsExchangeInt.stack( oResults.amxOpsExchangeInt( i3 ) );
      }
    OutputMatrixBin(
      sRunName + "OpsExchangeInt.dat", mxOpsExchangeInt
      );
    }
  }

void ConsoleNNHFOutputHandler1::Done( )
  {
  cout << "Computation " << sRunName << " ends." << endl;
  }

