
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Construction of a NSOF basis from a SOF basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ConstructBasisNsof.h"
#include "ArrayUtils.h"
#include "GetTranslationIndices.h"
#include "ConvertUniformBasisToArray.h"
#include "ConstructCoveringS.h"

pair< Basis, Array< Integer > >
ConstructBasisNsof(
  Basis const & basisSof,
// degree of the interpolating wavelet family
  Integer m
  )
  {
  Array< Integer > const aiDecSof = basisSof.GetSofDecomposition();
  Integer const nLevelsSof = aiDecSof.length();
  Array< BasisMultiIndex > amiNsof( 0 );
  Array< Integer > aiDecNsof( 0 );
  if( nLevelsSof > 0 )
    {
    Integer const siMin = basisSof( 0 ).si;
    Integer nS = aiDecSof( 0 );
    Array< BasisMultiIndex > amiS =
      GetRangeFromArray( basisSof.GetIndices(), 0, nS - 1 );
    if( nLevelsSof == 1 )
      {
      aiDecNsof = Array< Integer >( 1, nS );
      Basis const basisNsof( amiS );
      return pair< Basis, Array< Integer > >(
        basisNsof,
        aiDecNsof
        );
      }
    if( nS > 0 )
      {
      amiNsof = amiS;
      aiDecNsof = ConcatenateArrays(
        aiDecNsof,
        Array< Integer >( 1, nS )
        );
      }
    else
      {
      aiDecNsof = ConcatenateArrays(
        aiDecNsof,
        Array< Integer >( 1, 0 )
        );
      }
    Integer nD =
      ( nLevelsSof >= 2 )
      ? aiDecSof( 1 )
      : 0;
    Array< BasisMultiIndex > amiD =
      GetRangeFromArray( basisSof.GetIndices(), nS, nS + nD - 1 );
    amiNsof = ConcatenateArrays( amiNsof, amiD );
    aiDecNsof = ConcatenateArrays(
      aiDecNsof,
      Array< Integer >( 1, nD )
      );
    Integer nIndex = nS + nD;
    for( Integer nLevel = 2; nLevel < nLevelsSof; nLevel++ )
      {
      nD = aiDecSof( nLevel );
      Array< BasisMultiIndex > amiDNext =
        GetRangeFromArray( basisSof.GetIndices(), nIndex, nIndex + nD - 1 );
      ScalingIndex si = siMin + nLevel - 1;
      TranslationIndex tiSkMin =
        ( amiS.length() > 0 ) ? amiS( 0 ).ti : 0;
      TranslationIndex tiSkMax =
        ( amiS.length() > 0 ) ? amiS( amiS.length() - 1 ).ti : (-1);
      TranslationIndex tiDkMin =
        ( amiD.length() > 0 ) ? amiD( 0 ).ti : 0;
      TranslationIndex tiDkMax =
        ( amiD.length() > 0 ) ? amiD( amiD.length() - 1 ).ti : (-1);
      UniformBasis usfbNext =
        ConstructCoveringS( 
          si,
          tiSkMin,
          tiSkMax,
          tiDkMin,
          tiDkMax,
          m
          );
      Array< BasisMultiIndex > amiSNext =
        ConvertUniformBasisToArray( usfbNext );
      nS = amiSNext.length();
      amiNsof =
        ConcatenateArrays(
          amiNsof,
          ConcatenateArrays(
            amiSNext, amiDNext
            )
          );
      Array< Integer > aiDecNew( 2 );
      aiDecNew( 0 ) = nS;
      aiDecNew( 1 ) = nD;
      aiDecNsof =
        ConcatenateArrays(
          aiDecNsof,
          aiDecNew
          );
      nIndex += nD;
      amiS = amiSNext;
      amiD = amiDNext;
      }
    }
  Basis basisNsof( amiNsof );
  Array< Integer > const aiDecCheck =
    basisNsof.GetNsofDecomposition();
  if( ! EqualArrays( aiDecNsof, aiDecCheck ) )
    {
    OwnError( "ConstructBasisNsof: internal error" );
    return pair< Basis, Array< Integer > >(
      Basis(),
      Array< Integer >()
      );
    }
  return pair< Basis, Array< Integer > >(
    basisNsof,
    aiDecNsof
    );
  }

