
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Construction of S(k+1) basis function set
// covering given Sk and Dk basis function sets.

// Implementation file

// Copyright (C) 2004  Tommi Hynlnmaa

#include "ConstructCoveringS.h"
#include <algorithm>

#ifdef USE_NAMESPACES
using namespace std;
#endif

UniformBasis ConstructCoveringS(
  ScalingIndex siTarget,
  TranslationIndex tiSkMin,
  TranslationIndex tiSkMax,
  TranslationIndex tiDkMin,
  TranslationIndex tiDkMax,
  Integer nWaveletDegree
  )
  {
  TranslationIndex const tiMin =
    ( tiSkMin <= tiSkMax )
    ? ( ( tiDkMin <= tiDkMax )
        ? min( tiSkMin, tiDkMin )
        : tiSkMin )
    : ( ( tiDkMin <= tiDkMax )
        ? tiDkMin
        : 0 );
  TranslationIndex const tiMax =
    ( tiSkMin <= tiSkMax )
    ? ( ( tiDkMin <= tiDkMax )
        ? max( tiSkMax, tiDkMax )
        : tiSkMax )
    : ( ( tiDkMin <= tiDkMax )
        ? tiDkMax
        : ( -1 ) );
  if( tiMin <= tiMax )
    {
    TranslationIndex const tiStart = 2 * tiMin - nWaveletDegree;
    TranslationIndex const tiEnd = 2 * tiMax + nWaveletDegree;
    return UniformBasis( 0, siTarget, tiStart, tiEnd );
    }
  else
    {
    // uniform basis with empty range
    return UniformBasis( 0, 0, 0, -1 );
    }
  }
