
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Construction of a uniform scaling function basis
// covering given NSOF basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ConstructScfBasis.h"
#include "ConstructCoveringS.h"

UniformBasis ConstructScfBasis(
  Basis const & basisNsof,
  Array< Integer > const & aiDecNsof,
  Integer nWaveletDegree
  )
  {
  Integer const nLevels = aiDecNsof.length();
  if( nLevels < 1 )
    {
    OwnError( "ConstructScfBasis: too few components in basis" );
    return UniformBasis();
    }
  if( nLevels == 1 )
    {
    Integer const nBasis = basisNsof.length();
    Integer const nSk = aiDecNsof( 0 );
    TranslationIndex const tiSkMin =
      basisNsof( nBasis - nSk ).ti;
    TranslationIndex const tiSkMax =
      basisNsof( nBasis - 1 ).ti;
    ScalingIndex const si = basisNsof.GetMaxResolution();
    return UniformBasis( 0, si, tiSkMin, tiSkMax );
    }
  Integer const nBasis = basisNsof.length();
  Integer const nSk = aiDecNsof( nLevels - 2 );
  Integer const nDk = aiDecNsof( nLevels - 1 );
  if( nSk + nDk <= 0 )
    {
    OwnError( "ConstructScfBasis: error in decomposition" );
    return UniformBasis();
    }
  TranslationIndex const tiSkMin =
    basisNsof( nBasis - nDk - nSk ).ti;
  TranslationIndex const tiSkMax =
    basisNsof( nBasis - nDk - 1 ).ti;
  TranslationIndex const tiDkMin =
    ( nDk > 0 ) ? basisNsof( nBasis - nDk ).ti : 0;
  TranslationIndex const tiDkMax =
    ( nDk > 0 ) ? basisNsof( nBasis - 1 ).ti : ( -1 );
  ScalingIndex const si = basisNsof.GetMaxResolution();
  return
    ConstructCoveringS(
      si + 1,
      tiSkMin, tiSkMax,
      tiDkMin, tiDkMax,
      nWaveletDegree
      );
  }
