
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Conversion of a vector in NSOF basis to nearest
// scaling function basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ConvertVectorFromNsofToNearestScfBasis.h"

ColumnVector ConvertVectorFromNsofToNearestScfBasis(
  WaveletFilters const & wv,
  Basis const & basis_nsof,
  Array< Integer > const & aiDecNsof,
  UniformBasis const & usfb,
  ColumnVector vNsof
  )
  {
  Integer const nBasisNsof = basis_nsof.length();
  Integer const nUsfb = usfb.tiMax - usfb.tiMin + 1;
  Integer const nLevels = aiDecNsof.length();
  if( nLevels == 1 )
    {
    ColumnVector vResult( nUsfb, 0.0 );
    for( Integer i = 0; i < nBasisNsof; i++ )
      {
      BasisMultiIndex const mi = basis_nsof( i );
      Integer const iIndex = mi.ti - usfb.tiMin;
      if( iIndex < 0 || iIndex >= nUsfb )
        {
        OwnError( "ConvertVectorFromNsofToNearestScfBasis: error in bases" );
        return ColumnVector( );
        }
      vResult( iIndex ) = vNsof( i );
      }
    return vResult;
    }
  ColumnVector vResult( nUsfb, 0.0 );
  Integer ind_nsof_prev = 
    nBasisNsof - aiDecNsof( nLevels - 2 ) - aiDecNsof( nLevels - 1 );
  for( Integer i = 0; i < nUsfb; i++ )
    {
    Integer wi = i + usfb.tiMin;
    if( wi % 2 == 0 )
      {
      Integer wi_half = wi / 2;
      for( Integer j = 0; j < aiDecNsof( nLevels - 2 ); j++ )
        {
        Integer wi2 = basis_nsof( ind_nsof_prev + j ).ti;
        Integer mu = 2 * ( wi_half - wi2 );
        if( mu >= -wv.nM && mu <= wv.nM )
          {
          vResult( i ) +=
            wv.vH( mu + wv.nM ) * vNsof( ind_nsof_prev + j );
          }
        }
      for( Integer j = 0; j < aiDecNsof( nLevels - 1 ); j++ )
        {
        Integer wi2 = basis_nsof( 
          ind_nsof_prev + aiDecNsof( nLevels - 2 ) + j 
          ).ti;
        Integer mu = 2 * ( wi_half - wi2 );
        if( mu >= -wv.nM && mu <= wv.nM )
          {
          vResult( i ) +=
            wv.vG( mu + wv.nM ) 
            * vNsof( ind_nsof_prev + aiDecNsof( nLevels - 2 ) + j );
          }
        }
      }
    else
      {
      Integer wi_half = ( wi - 1 )/ 2;
      for( Integer j = 0; j < aiDecNsof( nLevels - 2 ); j++ )
        {
        Integer wi2 = basis_nsof( ind_nsof_prev + j ).ti;
        Integer mu = 2 * ( wi_half - wi2 ) + 1;
        if( mu >= -wv.nM && mu <= wv.nM )
          {
          vResult( i ) +=
            wv.vH( mu + wv.nM ) * vNsof( ind_nsof_prev + j );
          }
        }
      for( Integer j = 0; j < aiDecNsof( nLevels - 1 ); j++ )
        {
        Integer wi2 = basis_nsof( 
          ind_nsof_prev + aiDecNsof( nLevels - 2 ) + j 
          ).ti;
        Integer mu = 2 * ( wi_half - wi2 ) + 1;
        if( mu >= -wv.nM && mu <= wv.nM )
          {
          vResult( i ) +=
            wv.vG( mu + wv.nM ) 
            * vNsof( ind_nsof_prev + aiDecNsof( nLevels - 2 ) + j );
          }
        }
      }
    }
  return vResult;
  }
