
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Conversion from a vector from a scaling function basis
// to a standard basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ConvertVectorFromScfToStandardBasis.h"
#include "NTwoToIntegerPower.h"

static inline
Real ComputeComponentS(
  WaveletFilters const & wv,  
  ScalingIndex k,
  TranslationIndex i,
  UniformBasis const & usfb,
  ColumnVector v
  )
  {
  Integer const z = 
    NTwoToIntegerPower( usfb.si - k - 1 );
  Integer const j = 2 * z * i;
  if( j >= usfb.tiMin && j <= usfb.tiMax )
    {
    return v( j - usfb.tiMin );
    }
  else
    {
    return 0.0;
    }
  }

static inline
Real ComputeComponentD(
  WaveletFilters const & wv,
  ScalingIndex k,
  TranslationIndex i,
  UniformBasis const & usfb,
  ColumnVector v
  )
  {
  Integer const z = 
    NTwoToIntegerPower( usfb.si - k - 1 );
  Real result = 0.0;
  for( Integer nu = -wv.nM; nu <= wv.nM; nu++ )
    {
    Integer j = z * ( 2 * i + nu );
    if( j >= usfb.tiMin && j <= usfb.tiMax )
      {
      result += wv.vGt( nu + wv.nM ) * v( j - usfb.tiMin );
      }
    }
  return result;
  }

static inline
Real ComputeComponent(
  WaveletFilters const & wv,
  BasisMultiIndex alpha,
  UniformBasis const & usfb,
  ColumnVector v
  )
  {
  return
    ( alpha.wt == 0 )
    ? ComputeComponentS( wv, alpha.si, alpha.ti,
                         usfb, v )
    : ComputeComponentD( wv, alpha.si, alpha.ti,
                         usfb, v );
  }

ColumnVector ConvertVectorFromScfToStandardBasis(
  WaveletFilters const & wv,
  Basis const & basis,
  UniformBasis const & usfb,
  ColumnVector v
  )
  {
  Integer const n = basis.length();
  ColumnVector result( n );
  for( Integer i = 0; i < n; i++ )
    {
    result( i ) = ComputeComponent(
      wv, basis( i ), usfb, v
      );
    }
  return result;
  }
