
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Conversion of a vector from a standard basis
// to a NSOF basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "ConvertVectorToNsof.h"
#include "OwnError.h"

ColumnVector ConvertVectorToNsof(
  WaveletFilters const & wv,
  Basis const & basisnsof,
  Array< Integer > const & aiDecNsof,
  ColumnVector v
  )
  {
  Integer const nBasisNsof = basisnsof.length();
  Integer const nLevels = aiDecNsof.length();
  if( nLevels == 1 )
    {
    // only S part
    return v;
    }
  ColumnVector vNsof( nBasisNsof );
  vNsof.fill( 0.0 );
  Integer ind_nsof = 0;
  Integer ind_sof = 0;
  Integer ind_nsof_prev = 0;
  for( Integer lvl = 0; lvl < nLevels; lvl++ )
    {
    if( ( lvl == 0 ) || ( lvl % 2 == 1 ) )
      {
      // S_kmin and Dk parts
      for( Integer i = 0; i < aiDecNsof( lvl ); i++ )
        {
        vNsof( ind_nsof + i ) = v( ind_sof + i );
        }
      ind_sof += aiDecNsof( lvl );
      }
    else
      {
      // S_k parts for k > k_min
      for( Integer i = 0; i < aiDecNsof( lvl ); i++ )
        {
        BasisMultiIndex beta = basisnsof( ind_nsof + i );
        if( beta.wt != 0 )
          {
          OwnError( "ConvertVectorToNsof: error in basis" );
          return ColumnVector();
          }
        Integer wi = beta.ti;
        if( wi % 2 == 0 )
          {
          Integer wi_half = wi / 2;
          for( Integer j = 0; j < aiDecNsof( lvl - 2 ); j++ )
            {
            Integer wi2 = basisnsof( ind_nsof_prev + j ).ti;
            Integer mu = 2 * ( wi_half - wi2 );
            if( mu >= -wv.nM && mu <= wv.nM )
              {
              vNsof( ind_nsof + i ) +=
                wv.vH( mu + wv.nM ) * vNsof( ind_nsof_prev + j );
              }
            }
          for( Integer j = 0; j < aiDecNsof( lvl - 1 ); j++ )
            {
            Integer wi2 = basisnsof( 
              ind_nsof_prev + aiDecNsof( lvl - 2 ) + j 
              ).ti;
            Integer mu = 2 * ( wi_half - wi2 );
            if( mu >= -wv.nM && mu <= wv.nM )
              {
              vNsof( ind_nsof + i ) +=
                wv.vG( mu + wv.nM ) 
                * vNsof( ind_nsof_prev + aiDecNsof( lvl - 2 ) + j );
              }
            }
          }
        else
          {
          Integer wi_half = ( wi - 1 )/ 2;
          for( Integer j = 0; j < aiDecNsof( lvl - 2 ); j++ )
            {
            Integer wi2 = basisnsof( ind_nsof_prev + j ).ti;
            Integer mu = 2 * ( wi_half - wi2 ) + 1;
            if( mu >= -wv.nM && mu <= wv.nM )
              {
              vNsof( ind_nsof + i ) +=
                wv.vH( mu + wv.nM ) * vNsof( ind_nsof_prev + j );
              }
            }
          for( Integer j = 0; j < aiDecNsof( lvl - 1 ); j++ )
            {
            Integer wi2 = basisnsof( 
              ind_nsof_prev + aiDecNsof( lvl - 2 ) + j 
              ).ti;
            Integer mu = 2 * ( wi_half - wi2 ) + 1;
            if( mu >= -wv.nM && mu <= wv.nM )
              {
              vNsof( ind_nsof + i ) +=
                wv.vG( mu + wv.nM ) 
                * vNsof( ind_nsof_prev + aiDecNsof( lvl - 2 ) + j );
              }
            }
          }
        }      
      ind_nsof_prev = ind_nsof;
      }
    ind_nsof += aiDecNsof( lvl );
    }
  return vNsof;
  }
