
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of hydrogenic orbitals
// with arbitary principal and angular momentum
// quantum numbers (and arbitrary atomic number)

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef DoHydrogenicSComputation_h
#define DoHydrogenicSComputation_h

#include "OctaveIncludes.h"
#include "OctaveComplexIncludes.h"
#include "WaveletFilters.h"
#include "Basis.h"
#include "UniformBasis.h"
#include "ChooseWavefunction.h"

enum EnumGeneralHydrogenicComputationError
{
  errhsNoError,
  errhsSingularROperator,
  errhsWavefunctionNotFound
};

struct GeneralHydrogenicComputationResults
  {
    WavefunctionInfo wfinfo;
    EnumGeneralHydrogenicComputationError errhs;
    ComplexColumnVector cvEigenvalues;
    ComplexMatrix cmxEigenvectors;
    ComplexColumnVector cvSortedEigenvalues;
    ComplexMatrix cmxSortedEigenvectors;
    Array< Integer > aiSortIndices;
    Matrix mxOpT;
    Matrix mxOpV;
    Matrix mxOpC;
    Matrix mxOpHamiltonian;
};

GeneralHydrogenicComputationResults DoGeneralHydrogenicComputation(
  WaveletFilters const & wv,
  Array< Integer > anDecSof,
  Basis const & basisNsof,
  Array< Integer > anDecNsof,
  bool fZPresentInBasis,
  UniformBasis const & usfbScfBasis,
  ColumnVector vD1Filter,
  ColumnVector vD2Filter,
  Integer nAtomicNumber,
  Integer nAngularMomentumQuantumNumber,
  Integer nDesiredNodes,
  Real rAntisymmThreshold,
  Real rDerivThreshold,
  Real rSignCheckPosition
  );

#endif
