
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Filtering values in a column vector.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "FilterValues.h"
#include "OwnError.h"

ColumnVector FilterValues(
  ColumnVector vValues,
  ColumnVector vFilter
  )
  {
  if( ( vFilter.length() <= 0 ) ||
      ( vFilter.length() % 2 == 0 ) )
    {
    OwnError( "FilterValues: error in filter" );
    return ColumnVector();
    }
  Integer const n = ( vFilter.length() - 1 ) / 2;
  Integer const nPoints = vValues.length();
  ColumnVector vResult( nPoints );
  vResult.fill( 0.0 );
  for( Integer i = 0; i < nPoints; i++ )
    {
    for( Integer index = -n; index <= n; index++ )
      {
      Integer j = i + index;
      if( j >= 0 && j < nPoints )
        {
        vResult( i ) += vFilter( n + index ) * vValues( j );
        }
      }
    }
  return vResult;
  }
