
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Construction of a range of basis multi-indices.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetBasisMultiIndexRange.h"

Array< BasisMultiIndex >
GetBasisMultiIndexRange(
  WaveletType wt,
  ScalingIndex si,
  TranslationIndex tiMin,
  TranslationIndex tiMax,
  bool fExcludeZero
  )
  {
  if( tiMin > tiMax )
    {
    return Array< BasisMultiIndex >();
    }
  Integer const nCount =
    fExcludeZero ? ( tiMax - tiMin ) : ( tiMax - tiMin + 1 );
  Array< BasisMultiIndex > ami( nCount );
  if( fExcludeZero && ( tiMin <= 0 && tiMax >= 0 ) )
    {
    for( TranslationIndex ti1 = tiMin; ti1 <= -1; ti1++ )
      {
      ami( ti1 - tiMin ) =
        BasisMultiIndex( wt, si, ti1 );
      }
    for( TranslationIndex ti2 = 1; ti2 <= tiMax; ti2++ )
      {
      ami( ti2 - tiMin - 1 ) =
        BasisMultiIndex( wt, si, ti2 );
      }
    }
  else
    {
    for( TranslationIndex ti = tiMin; ti <= tiMax; ti++ )
      {
      ami( ti - tiMin ) =
        BasisMultiIndex( wt, si, ti );
      }
    }
  return ami;
  }
