
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Extraction of a basis part from a NSOF basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetBasisPartNsof.h"
#include "ArrayUtils.h"
#include "OwnError.h"

// #define DEBUG_OUTPUT

#ifdef DEBUG_OUTPUT
#include <iostream.h>
#endif

BasisPart GetBasisPartNsof(
  Basis const & basisnsof,
  Array< Integer > aiDecNsof,
  ScalingIndex siMin,
  WaveletType wt,
  ScalingIndex si
  )
  {
#ifdef DEBUG_OUTPUT
  cout << "GetBasisPartNsof ENTER" << endl;
  cout << "  siMin = " << siMin << endl;
  cout << "  wt = " << wt << endl;
  cout << "  si = " << si << endl;
#endif
  Integer const nLevel =
    ( wt == 0 )
    ? ( 2 * ( si - siMin ) )
    : ( 2 * ( si - siMin ) + 1 );
  if( nLevel < 0 || nLevel >= aiDecNsof.length() )
    {
    OwnError( "GetBasisPartNsof: index out of range" );
    return BasisPart( 0, 0, Array< TranslationIndex >() );
    }
  Integer const itiBase =
    ( nLevel > 0 )
    ? SumArrayElementsInRange( aiDecNsof, Integer( 0 ), 0, nLevel - 1 )
    : 0;
  Integer const n = aiDecNsof( nLevel );
#ifdef DEBUG_OUTPUT
  cout << "  itiBase = " << itiBase << endl;
  cout << "  n = " << n << endl;
  cout << "  basisnsof.length() = " << basisnsof.length() << endl;
#endif
  if( itiBase < 0 || n < 0 || itiBase + n > basisnsof.length() )
    {
    OwnError( "GetBasisPartNsof: invalid basis decomposition" );
    return BasisPart( 0, 0, Array< TranslationIndex >() );
    }
  Array< TranslationIndex > ati( n );
  for( Integer iti = 0; iti < n; iti++ )
    {
    ati( iti ) = basisnsof( itiBase + iti ).ti;
    }
#ifdef DEBUG_OUTPUT
  cout << "GetBasisPartNsof EXIT" << endl;
#endif
  return BasisPart( wt, si, ati );
  }
