
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Wavelet filters for 8th degree interpolating wavelet family

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetFiltersInterp8.h"
#include "MinusOneToIntPower.h"

// Since h filter is symmetric around 0,
// only elements h_i for i >= 0 need to be given.
static Integer const nInterp8H = 7;
static Real const arInterp8H[ nInterp8H + 1 ] =
{
  1.0,
  1225.0/2048,
  0.0, 
  -245.0/2048,
  0.0,
  49.0/2048,
  0.0,
  -5.0/2048
};

WaveletFilters GetFiltersInterp8()
  {

  // Degree of the interpolating wavelet family
  Integer const nM = 8;

  // Filter h
  ColumnVector vH( 2 * nM + 1, 0.0 );
  vH( nM ) = arInterp8H[ 0 ];
  for( Integer i1 = 1; i1 <= nInterp8H; i1++ )
    {
    // Set h_i
    vH( nM + i1 ) = arInterp8H[ i1 ];
    // Set h_{-i} = h_i
    vH( nM - i1 ) = arInterp8H[ i1 ];
    }

  // Filter ht
  ColumnVector vHt( 2 * nM + 1, 0.0 );
  vHt( nM ) = 1.0;

  // Filter g
  ColumnVector vG( 2 * nM + 1, 0.0 );
  for( Integer i2 = -nM; i2 <= nM - 1; i2++ )
    {
    vG( nM + i2 + 1 ) = vHt( nM - i2 ) * MinusOneToIntPower( i2 + 1 );
    }

  // Filter gt
  ColumnVector vGt( 2 * nM + 1, 0.0 );
  for( Integer i3 = -nM; i3 <= nM - 1; i3++ )
    {
    vGt( nM + i3 + 1 ) = vH( nM - i3 ) * MinusOneToIntPower( i3 + 1 );
    }

  return WaveletFilters( nM, vH, vG, vHt, vGt );
  }
