
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Computation of a linear space of values.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetLinearSpace.h"
#include "OwnError.h"

ColumnVector GetLinearSpace(
  Real rMin,
  Real rMax,
  Integer n
  )
  {
  if( n < 0 )
    {
    OwnError( "GetLinearSpace: number of elements has to be nonnegative" );
    return ColumnVector();
    }
  else if( n == 0 )
    {
    return ColumnVector();
    }
  else if( n == 1 )
    {
    if( rMin != rMax )
      {
      OwnError( "GetLinearSpace: minimum and maximum have to be equal for a space of one element" );
      return ColumnVector();
      }
    return ColumnVector( 1, rMin );
    }
  else
    {
    ColumnVector vResult( n );
    Real const rStep = ( rMax - rMin ) / ( n - 1 );
    for( Integer i = 0; i < n; i++ )
      {
      vResult( i ) = rMin + i * rStep;
      }
    return vResult;
    }
  }
