
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Extraction of the part in nonnegative real axis
// from a function in uniform scaling function basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetNonnegativePart.h"
#include "ArrayUtils.h"
#include "OwnError.h"

pair< UniformBasis, ColumnVector >
GetNonnegativePart(
  pair< UniformBasis, ColumnVector > const & usfbFunction_vFunction
  )
  {
  UniformBasis const & usfbFunction =
    usfbFunction_vFunction.first;
  ColumnVector const & vFunction =
    usfbFunction_vFunction.second;
  if( usfbFunction.wt != 0 )
    {
    OwnError( "GetNonnegativePart: wrong basis function type" );
    return pair< UniformBasis, ColumnVector >();
    }
  if( usfbFunction.tiMin >= 0 )
    {
    return usfbFunction_vFunction;
    }
  else
    {
    Integer const n = usfbFunction.tiMax - usfbFunction.tiMin + 1;
    Integer const iStart = -usfbFunction.tiMin;
    if( iStart >= n )
      {
      return pair< UniformBasis, ColumnVector >(
        UniformBasis( 0, usfbFunction.si, 0, -1 ),
        ColumnVector()
        );
      }
    else
      {
      return pair< UniformBasis, ColumnVector >(
        UniformBasis( 0, usfbFunction.si, 0, usfbFunction.tiMax ),
        ColumnVector( GetRangeFromArray( vFunction, iStart, n - 1 ) )
        );
      }
    }
  }
