
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Getting sorted indices for a column vector.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "GetSortedIndicesForColumnVector.h"
#include "OwnError.h"
#include <cstdlib>

struct ColumnVectorSortEntry
  {
    Real r;
    Integer i;
};

static
int CompareEntries( void const * p1, void const * p2 )
  {
  ColumnVectorSortEntry const & oEntry1 =
    *( ( ColumnVectorSortEntry const * ) p1 );
  ColumnVectorSortEntry const & oEntry2 =
    *( ( ColumnVectorSortEntry const * ) p2 );
  return ( oEntry1.r > oEntry2.r ) - ( oEntry1.r < oEntry2.r );
  }

Array< Integer >
GetSortedIndicesForColumnVector(
  ColumnVector v
  )
  {
  Integer const n = v.length();
  if( n == 0 )
    {
    return Array< Integer >();
    }
  ColumnVectorSortEntry* aSortEntries =
    new ColumnVectorSortEntry[ n ];
  if( aSortEntries == NULL )
    {
    OwnError( "GetSortedIndicesForColumnVector: memory allocation error" );
    return Array< Integer >();
    }
  for( Integer iSort = 0; iSort < n; iSort++ )
    {
    aSortEntries[ iSort ].r = v( iSort );
    aSortEntries[ iSort ].i = iSort;
    }
  qsort(
    aSortEntries, 
    n, 
    sizeof( ColumnVectorSortEntry ),
    CompareEntries
    );
  Array< Integer > aiResult( n );
  for( Integer iSort = 0; iSort < n; iSort++ )
    {
    aiResult( iSort ) = aSortEntries[ iSort ].i;
    }
  delete [] aSortEntries;
  return aiResult;
  }
