
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Class for Helium HF computation

// Header file

// Copyright (C) 2004  Tommi Hynlnmaa

#ifndef HeliumHFComputation_h
#define HeliumHFComputation_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include "WaveletFilters.h"
#include "HFOrbitalComputationParams.h"
#include "UsfbTabulatedFunction.h"
#include "MotherIntegral.h"
#include "HFLogger.h"
#include "ComputeMatrixNorm.h"
#include "HFDefs.h"
#include "HeliumHFOutputHandler.h"
#include "Triplet.h"
#include <utility>

class HeliumHFComputation
  {
  protected:
    static Integer const nAtomicNumber = 2;
    static Integer const nOrbitals = 1;
    string sRunName;
    Array< Integer > anOrbitalOccupations;
    Integer nMaxIterations;
    Real rConvThreshold;
    ScalingIndex siScfResolution;
    EnumMatrixNormType nConvNormType;
    WaveletFilters wv;
    ColumnVector vD1Filter;
    ColumnVector vD2Filter;
    HFBasisData hfbasisdata;
    HFOrbitalComputationParams hforbparams1s;
    UsfbTabulatedFunction usfbfnScf;
    MotherIntegral motherintegral;
    int nExitCode;
    HeliumHFOutputHandler* pOutputHandler;
    HFLogger* pLogger;

  public:

    HeliumHFComputation( );

    virtual void SetRunName( string sRunName1 );
    virtual void SetParameters(
      Integer const nMaxIterations1,
      Real rConvThreshold1,
      ScalingIndex siScfResolution1,
      EnumMatrixNormType nConvNormType1
      );
    virtual pair< Basis, Integer > GetBasis( ) const = 0;
    virtual HFOrbitalComputationParams GetOrbitalParams( ) const = 0;
    virtual void InitFilters( ) = 0;
    virtual void InitBasis( );
    virtual void InitMotherScalingFunction( );
    virtual void InitOrbitalParams( );
    virtual void Initialize( );
    virtual HeliumHFOutputHandler* GetOutputHandler( ) const = 0;
    virtual HFLogger* GetLogger( ) const = 0;
    virtual Triplet< Real, ColumnVector, ColumnVector > XComputeAtomEnergy(
      HeliumHFComputationResults const & oResults
      ) const;
    virtual HeliumHFComputationResults DoComputation( ) const;
    virtual void Run( );
    virtual int GetExitCode( ) const;
    virtual ~HeliumHFComputation( );
  };

#ifdef OCTAVE_DL_COMPILATION
#include "HeliumHFComputation.cc"
#endif

#endif

