
// Distributed under GNU Lesser General Public License
// See file LICENSE


// class MotherIntegral

// Class for integral function of the mother scaling function.

// Header file without implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef MotherIntegral_h
#define MotherIntegral_h

#include "CommonTypes.h"
#include "UsfbTabulatedFunction.h"
#include "WaveletFilters.h"

class MotherIntegral 
  {
    private:

  UsfbTabulatedFunction usfbfnInt;
  Real rIntAtInf;

    public:

  MotherIntegral( )
    {
    }

  MotherIntegral( 
    UsfbTabulatedFunction const & usfbfnInt1,
    Real rIntAtInf1
    )
    : usfbfnInt( usfbfnInt1 ), rIntAtInf( rIntAtInf1 )
    {
    }

  Real GetValueAt( Integer i ) const
    {
    if( i < usfbfnInt.usfb.tiMin )
      {
      // assume that value at negative infinity is zero
      return 0.0;
      }
    else if( i > usfbfnInt.usfb.tiMax )
      {
      return rIntAtInf;
      }
    else
      {
      return usfbfnInt.vValues( 
        i - usfbfnInt.usfb.tiMin
        );
      }
    }

  UsfbTabulatedFunction GetFunction() const
      {
      return usfbfnInt;
      }

  Real GetValueAtInf() const
    {
    return rIntAtInf;
    }

};

#endif
