
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Class for HF computation neglecting 
// nondiagonal Lagrange multipliers

// Header file

// Copyright (C) 2004  Tommi Hynlnmaa

#ifndef NNHFComputation_h
#define NNHFComputation_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include "WaveletFilters.h"
#include "HFOrbitalComputationParams.h"
#include "UsfbTabulatedFunction.h"
#include "MotherIntegral.h"
#include "HFLogger.h"
#include "ComputeMatrixNorm.h"
#include "HFDefs.h"
#include "NNHFOutputHandler.h"
#include "Triplet.h"
#include <utility>

class NNHFComputation
  {
  protected:
    string sRunName;
    Integer nMaxIterations;
    Real rConvThreshold;
    ScalingIndex siScfResolution;
    EnumMatrixNormType nConvNormType;
    WaveletFilters wv;
    ColumnVector vD1Filter;
    ColumnVector vD2Filter;
    HFAtomicData hfatomicdata;
    HFBasisData hfbasisdata;
    Array< HFOrbitalComputationParams > aOrbitalParams;
    UsfbTabulatedFunction usfbfnScf;
    MotherIntegral motherintegral;
    int nExitCode;
    NNHFOutputHandler* pOutputHandler;
    HFLogger* pLogger;

  public:

    NNHFComputation( );

    virtual void SetRunName( string sRunName1 );
    virtual void SetParameters(
      Integer const nMaxIterations1,
      Real rConvThreshold1,
      ScalingIndex siScfResolution1,
      EnumMatrixNormType nConvNormType1
      );
    virtual HFAtomicData GetAtomicData( ) const = 0;
    virtual pair< Basis, Integer > GetBasis( ) const = 0;
    virtual Array< HFOrbitalComputationParams > GetOrbitalParams( ) const = 0;
    virtual void InitFilters( ) = 0;
    virtual void InitBasis( );
    virtual void InitMotherScalingFunction( );
    virtual void InitAtomicData( );
    virtual void InitOrbitalParams( );
    virtual void Initialize( );
    virtual NNHFOutputHandler* GetOutputHandler( ) const = 0;
    virtual HFLogger* GetLogger( ) const = 0;
    virtual Triplet< Real, ColumnVector, ColumnVector > XComputeAtomEnergy(
      NondiagNeglHFComputationResults const & oResults
      ) const;
    virtual NondiagNeglHFComputationResults DoComputation( ) const;
    virtual void Run( );
    virtual int GetExitCode( ) const;
    virtual ~NNHFComputation( );
  };

#ifdef OCTAVE_DL_COMPILATION
#include "NNHFComputation.cc"
#endif

#endif

