
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Subclass of NNHFComputation with parametrization

// Header file

// Copyright (C) 2004  Tommi Hynlnmaa

#ifndef ParametrizedNNHFComputation_h
#define ParametrizedNNHFComputation_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include "NNHFComputation.h"

// Constraints:
// AtomicDataFactory contains a member function
//   static HFAtomicData GetAtomicData( )
// OrbitalParameterFactory contains a member function
//   static Array< HFOrbitalComputationParams > GetOrbitalParameters( )
// FilterInitializer contains member functions
//   static WaveletFilters GetWaveletFilters( )
//   static ColumnVector GetD1Filter( )
//   static ColumnVector GetD2Filter( )
// BasisFactory contains a member function
//   static pair< Basis, Integer > GetBasis( )
// OutputHandler is a subclass of NNHFOutputHandler
// Logger is a subclass of HFLogger
template< 
  class AtomicDataFactory, 
  class OrbitalParameterFactory,
  class FilterInitializer,
  class BasisFactory,
  class OutputHandler, class Logger >
class ParametrizedNNHFComputation : public NNHFComputation
  {
  public:
    virtual HFAtomicData GetAtomicData( ) const;
    virtual Array< HFOrbitalComputationParams > GetOrbitalParams( ) const;
    virtual pair< Basis, Integer > GetBasis( ) const;
    virtual void InitFilters( );
    virtual OutputHandler* GetOutputHandler( ) const;
    virtual Logger* GetLogger( ) const;
  };

#ifdef INCLUDE_TEMPLATE_DEFINITIONS
#include "ParametrizedNNHFComputation.cc"
#endif

#endif
