
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Fourth order Runge-Kutta integration for a real vector
// of values.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "RK4IntegrateValues.h"

ColumnVector
RK4IntegrateValues( ColumnVector vValues, Real rStep )
  {
  Integer const nPoints = vValues.length();
  Integer const n =
    ( nPoints % 2 == 0 ) ? ( nPoints / 2 ) : ( nPoints + 1 ) / 2;
  ColumnVector vIntegral( n, 0.0 );
  if( n == 0 )
    {
    return vIntegral;
    }
  vIntegral( 0 ) = 0.0;
  for( Integer i = 1; i < n; i++ )
    {
    Real const rK1 = vValues( 2 * i - 2 );
    Real const rK2 = vValues( 2 * i - 1 );
    Real const rK4 = vValues( 2 * i );
    vIntegral( i ) =
      vIntegral( i - 1 ) + rK1 / 6 + 2 * ( rK2 / 3 ) + rK4 / 6;
    }
  return rStep * vIntegral;
  }
