
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Refining vector in uniform standard function basis
// to a given resolution level.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "RefineVectorInUsfb.h"
#include "Triplet.h"
#include "OwnError.h"
#include "RefineVectorInUsfbToNextLevel.h"

// #define DEBUG_OUTPUT

#ifdef DEBUG_OUTPUT
#include <iostream>
#endif

pair< UniformBasis, ColumnVector >
RefineVectorInUsfb(
  WaveletFilters const & wv,
  ScalingIndex siTarget,
  pair< UniformBasis, ColumnVector > const & usfb_vector
  )
  {
#ifdef DEBUG_OUTPUT
  cout << "RefineVectorInUsfb ENTER" << endl;
#endif
  if( siTarget < usfb_vector.first.si )
    {
    // error in arguments
    OwnError( "RefineVectorInUsfb: invalid target resolution" );
    return pair< UniformBasis, ColumnVector >();
    }
  else if( siTarget == usfb_vector.first.si )
    {
#ifdef DEBUG_OUTPUT
    cout << "RefineVectorInUsfb EXIT 1" << endl;
#endif
    return usfb_vector;
    }
  else
    {
    pair< UniformBasis, ColumnVector > usfb_vectorPrev =
      RefineVectorInUsfb(
        wv, siTarget - 1, usfb_vector
        );
    Triplet< ColumnVector, TranslationIndex, TranslationIndex > 
      v_ti_tiRefined
      = RefineVectorInUsfbToNextLevel(
        wv,
        Triplet< ColumnVector, TranslationIndex, TranslationIndex >(
          usfb_vectorPrev.second,
          usfb_vectorPrev.first.tiMin,
          usfb_vectorPrev.first.tiMax
          )
        );
#ifdef DEBUG_OUTPUT
    cout << "RefineVectorInUsfb EXIT 2" << endl;
#endif
    return pair< UniformBasis, ColumnVector >(
      UniformBasis(
        0, siTarget, v_ti_tiRefined.second, v_ti_tiRefined.third
        ),
      v_ti_tiRefined.first
      );
    }
  }
