
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Refining a vector in uniform scaling function basis
// to next resolution level.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "RefineVectorInUsfbToNextLevel.h"

Triplet< ColumnVector, TranslationIndex, TranslationIndex >
RefineVectorInUsfbToNextLevel(
  WaveletFilters const & wv,
  Triplet< ColumnVector, TranslationIndex, TranslationIndex > const & v_ti_ti
  )
  {
  ColumnVector const v = v_ti_ti.first;
  TranslationIndex const tiMin = v_ti_ti.second;
  TranslationIndex const tiMax = v_ti_ti.third;
  Integer const tiMinNext = 2 * tiMin - wv.nM;
  Integer const tiMaxNext = 2 * tiMax + wv.nM;
  Integer const n = tiMaxNext - tiMinNext + 1;
  ColumnVector vNext( n, 0.0 );
  for( Integer j = 0; j < n; j++ )
    {
    Integer const m = j + tiMinNext;
    Integer const offset = ( m % 2 != 0 ) ? 1 : 0;
    // Subtracting offset not really necessary
    // since this is integer division.
    Integer const i = ( m - offset ) / 2;
    for( Integer mu = - ( wv.nM / 2 ); mu <= wv.nM / 2; mu++ )
      {
      Integer const tiPrev = i - mu;
      Integer const nu = 2 * mu + offset;
      if( tiPrev >= tiMin && tiPrev <= tiMax &&
          nu >= -wv.nM && nu <= wv.nM )
        {
        vNext( j ) += wv.vH( nu + wv.nM ) * v( tiPrev - tiMin );
        }
      }
    }
  return 
    Triplet< ColumnVector, TranslationIndex, TranslationIndex >
    ( vNext, tiMinNext, tiMaxNext );
  }
