
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Removing the highest resolution level from a NSOF basis.

// Implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "RemoveTopFromNsofBasis.h"
#include "ArrayUtils.h"
#include "OwnError.h"

// #define DEBUG_OUTPUT

#ifdef DEBUG_OUTPUT
#include <iostream>
#endif

pair< Basis, Array< Integer > >
RemoveTopFromNsofBasis(
  pair< Basis, Array< Integer > > basis_decNsof
  )
  {
#ifdef DEBUG_OUTPUT
  cout << "RemoveTopFromNsofBasis ENTER" << endl;
#endif
  Basis const & basis = basis_decNsof.first;
  Array< Integer > const & anDecNsof = basis_decNsof.second;
  Integer const nComponents = anDecNsof.length();
  if( nComponents <= 2 )
    {
    OwnError( "RemoveTopFromNsofBasis: too few parts in basis" );
    return pair< Basis, Array< Integer > >(
      Basis(), Array< Integer >()
      );
    }
  // parts 0,...,nComponents - 2 - 1
  Array< Integer > anDecNew =
    GetRangeFromArray( anDecNsof, 0, nComponents - 3 );
  Integer const nBasisNew =
    SumArrayElements( anDecNew, Integer( 0 ) );
  if( nBasisNew < 0 )
    {
    OwnError( "RemoveTopFromNsofBasis: invalid basis decomposition" );
    return pair< Basis, Array< Integer > >(
      Basis(), Array< Integer >()
      );
    }
  Array< BasisMultiIndex > amiNew( nBasisNew );
  for( Integer i = 0; i < nBasisNew; i++ )
    {
    amiNew( i ) = basis( i );
    }
#ifdef DEBUG_OUTPUT
  cout << "RemoveTopFromNsofBasis EXIT" << endl;
#endif
  return
    pair< Basis, Array< Integer > >(
      Basis( amiNew ), anDecNew
      );
  }
