
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Sorting wavefunctions in ascending order by
// (real part of) eigenvalues.

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "SortWavefunctions.h"
#include "GetSortedIndicesForColumnVector.h"

Triplet< ComplexColumnVector, ComplexMatrix, Array< Integer > >
SortWavefunctions(
  ComplexColumnVector cvEigenvalues,
  ComplexMatrix cmxEigenvectors
  )
  {
  Array< Integer > const aiSorted =
    GetSortedIndicesForColumnVector( real( cvEigenvalues ) );
  Integer const nVal = cvEigenvalues.length();
  ComplexColumnVector cvSortedEigenvalues( nVal, 0.0 );
  for( Integer i = 0; i < nVal; i++ )
    {
    cvSortedEigenvalues( i ) = cvEigenvalues( aiSorted( i ) );
    }
  Integer const nRows = cmxEigenvectors.rows();
  Integer const nColumns = cmxEigenvectors.columns();
  ComplexMatrix cmxSortedEigenvectors( nRows, nColumns );
  for( Integer i = 0; i < nColumns; i++ )
    {
    cmxSortedEigenvectors.insert(
      cmxEigenvectors.column( aiSorted( i ) ),
      0,
      i
      );
    }
  return Triplet< ComplexColumnVector, ComplexMatrix, Array< Integer > >(
    cvSortedEigenvalues,
    cmxSortedEigenvectors,
    aiSorted
    );
  }
