
// Distributed under GNU Lesser General Public License
// See file LICENSE


// class TabulatedFunction

// A real function represented as y values tabulated
// at evenly spaced x values.

// Header file without implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef TabulatedFunction_h
#define TabulatedFunction_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"
#include <cmath>

class TabulatedFunction 
  {
  protected:

  ColumnVector x;
  ColumnVector y;
  Integer points;
  Real step;

  public:

  TabulatedFunction()
    : x( 1 ), y( 1 ), points( 0 ), step( 0.0 )
    {
    x( 0 ) = 0.0;
    y( 0 ) = 0.0;
    }

  TabulatedFunction( ColumnVector x1,
		     ColumnVector y1,
		     Integer points1,
		     Real step1 )
    : x( x1 ), y( y1 ), points( points1 ), step( step1 )
    {
    }

  Real ValueAt( Real xarg ) const
    // Computation of value of the function at a point
    // with linear interpolation.
    // Outside range of tabulation function is set to 0.
    {
    Real result = 0.0;
    if( points > 0 && 
        ( xarg >= x( 0 ) && xarg <= x( x.length() - 1 ) ) )
      {
      Integer const i = Integer( floor( ( xarg - x( 0 ) ) / step ) );
      if( i >= 0 && i < points - 1 ) 
	{
	Real const x1 = x( i );
	Real const x2 = x( i + 1 );
	Real const y1 = y( i );
	Real const y2 = y( i + 1 );
	result = ( ( x2 - xarg ) * y1 + ( xarg - x1 ) * y2 ) / ( x2 - x1 );
	}
      }
    return result;
    }

  ColumnVector GetX() const
      {
      return x;
      }

  ColumnVector GetY() const
      {
      return y;
      }

  Integer GetPoints() const
      {
      return points;
      }

  Real GetStep() const
      {
      return step;
      }

  };

#endif
