
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Class UniformBasis

// Basis consisting of wavelets or scaling functions
// of same type, same resolution k, and translation index
// values of uniterrupted range i_min,...,i_max.

// Header file without implementation file

// Copyright (C) 2003  Tommi Hynlnmaa

#ifndef UniformBasis_h
#define UniformBasis_h

#include "OctaveIncludes.h"
#include "CommonTypes.h"

class UniformBasis
  {
  public:

  WaveletType wt;
  ScalingIndex si;
  TranslationIndex tiMin;
  TranslationIndex tiMax;

  UniformBasis()
    : wt( 0 ), si( 0 ), tiMin( 0 ), tiMax( -1 )
      {
      }

  UniformBasis( WaveletType wt1,
                ScalingIndex si1,
                TranslationIndex tiMin1,
                TranslationIndex tiMax1 ) : 
    wt( wt1 ),
    si( si1 ),
    tiMin( tiMin1 ),
    tiMax( tiMax1 )
      {
      }

  };

inline
bool operator == ( 
  UniformBasis const & usfb1, UniformBasis const & usfb2
  )
  {
  return
    usfb1.wt == usfb2.wt &&
    usfb1.si == usfb2.si &&
    usfb1.tiMin == usfb2.tiMin &&
    usfb1.tiMax == usfb2.tiMax;
  }

inline
bool operator != ( 
  UniformBasis const & usfb1, UniformBasis const & usfb2
  )
  {
  return
    usfb1.wt != usfb2.wt ||
    usfb1.si != usfb2.si ||
    usfb1.tiMin != usfb2.tiMin ||
    usfb1.tiMax != usfb2.tiMax;
  }

#endif
