
// Distributed under GNU Lesser General Public License
// See file LICENSE


// Vector utility functions.

// Header file

// Copyright (C) 2003  Tommi Hynlnmaa

#include "VectorUtils.h"
#include "OwnError.h"

ColumnVector
InsertElementToColumnVector(
  ColumnVector v,
  Integer i,
  Real r
  )
  {
  Integer const nLength = v.length();
  if( i < 0 || i > nLength )
    {
    OwnError( "InsertElementToColumnVector: index out of range" );
    return ColumnVector();
    }
  return
    ( nLength == 0 )
    ? ColumnVector( 1, r )
    : ( ( i > 0 ) 
        ? v.extract( 0, i - 1 )
        .stack( ColumnVector( 1, r ) )
        .stack( v.extract( i, nLength - 1 ) )
        : ColumnVector( 1, r ).stack( v ) );
  }

Real GetColumnVectorInfinityNorm( ColumnVector v )
  {
  Integer const len = v.length();
  Real nrm = 0;
  if( len > 0 )
    {
    nrm = fabs( v( 0 ) );
    for( Integer i = 1; i < len; i++ )
      {
      if( fabs( v( i ) ) > nrm )
        {
        nrm = fabs( v( i ) );
        }
      }
    }
  return nrm;
  }
